function [m,list] = assign(m,varargin)
%
% ASSIGN  Assign or reassign parameters and/or steady states.
%
% Syntax:
%   [m,list] = assign(m,d)
%   [m,list] = assign(m,'name1',value1,'name2',value2,...)
%   [m,list] = assign(m,names,values)
%   [m,list] = assign(m,values)
% Arguments:
%   m model; list cellstr; d struct; names cellstr; values numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

nalt = size(m.assign,3);

invalid = {};
if nargin == 1 % assign(m)
  return
elseif nargin == 2 && isnumeric(varargin{1}) % assign(m,[1,2])
  list = 1 : length(m.name);
  if sum(size(varargin{1}) > 1) == 1
    value = vech(varargin{1});
    value(1,:,2:nalt) = value(1,:,ones([1,nalt-1]));
  else
    value = permute(varargin{1},[3,1,2]);
  end
  if all(size(value) == size(m.assign)), m.assign(1,:,:) = value(1,:,:);
    else, invalid = m.name; end
elseif nargin > 2 && iscellstr(varargin(1:2:end)) % assign(m,'a',1,'b',2)
  list = findnames(m.name,varargin(1:2:end));
  tmp = varargin(2:2:end);
  tmp(isnan(list)) = [];
  list(isnan(list)) = [];
  for i = 1 : length(tmp)
    value_(tmp{i},list(i));
  end
elseif nargin == 2 && isstruct(varargin{1}) % assign(m,d)  
  dbase = fieldnames(varargin{1});
  list = [];
  if length(dbase) < length(m.name)
    for i = 1 : length(dbase)
      tmp = find(strcmp(m.name,dbase{i}));
      if isempty(tmp), continue, end
      list(end+1) = tmp;
      value_(varargin{1}.(dbase{i}),tmp);
    end
  else
    for i = 1 : length(m.name)
      try, value_(varargin{1}.(m.name{i}),i);
        catch, continue, end;
      list(end+1) = i;
    end
  end
elseif nargin == 3 && iscellstr(varargin{1}) && isnumeric(varargin{2}) % assign(m,{'a','b'},[1,2])
  index = findnames(m.name,varargin{1});
  value = varargin{2};
  if size(value,3) == 1 && nalt > 1, value = value(:,:,ones([1,nalt]));
    elseif size(value,3) ~= nalt, invalid = varargin{1}; end
  m.assign(1,index,:) = value;
else
  error('Incorrect type of input argument(s).');
end

if ~isempty(invalid), error_(46,unique(invalid)); end
if nargout > 1, list = m.name(list); end

  function value_(value,index) % nested function ------------------------------------------------------------

  value = permute(value(:),[3,2,1]);
  if length(value) == nalt, value = value(1,ones([1,length(index)]),:);
    elseif length(value) == 1, value = value(1,ones([1,length(index)]),ones([1,nalt]));
    else invalid(end+(1:length(index))) = m.name(index); return, end
  m.assign(1,index,:) = value;

  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------