function [flag,mval,list] = chksstate(m,varargin)
%
% CHKSSTATE  Evaluate model equations with currently assigned steady state.
%
% [flag,mval,list] = chksstate(m,...)
% Arguments:
%   flag logical, mval cell, list cellstr, m model
% Options:
%   'tolerance' numeric (eps^(2/3))
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'tolerance',getrealsmall(),...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

t = m.tzero;
mint = 1 - t;
maxt = size(m.occur,3) - t;
nalt = size(m.assign,3);

nm = sum(m.eqtntype == 1);
nt = sum(m.eqtntype == 2);

tvec = mint : maxt;
flag = false([1,nalt]);
list = cell([1,nalt]);
mval = {nan([nm,nalt]),nan([nt,nalt])};

for ialt = 1 : nalt
  x = trendarray_(m,1:sum(m.nname),tvec,true,ialt);
  x = shiftdim(x,-1);

  expr = cell([1,2]);
  expr{1} = ['[',vech(transpose(char(m.eqtnF(m.eqtntype == 1)))),']'];
  expr{2} = ['[',vech(transpose(char(m.eqtnF(m.eqtntype == 2)))),']'];
  expr{1}(strfind(expr{1},' ')) = '';
  expr{2}(strfind(expr{2},' ')) = '';

  if nm > 0, mval{1}(:,ialt) = eval(expr{1}); end
  mval{2}(:,ialt) = eval(expr{2});
  index = abs([mval{1}(:,ialt);mval{2}(:,ialt)]) <= options.tolerance;
  flag(ialt) = all(index);

  if nargout > 1, list{ialt} = m.eqtn(~index); end
end

if nalt == 1 && nargout > 1, list = list{1}; end

end % of primary function -----------------------------------------------------------------------------------