function m = expand(m,k)
%
% EXPAND  Forward expansion of model solution to t+k.
%
% m = expand(m,k)
% Arguments:
%   m model, k numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end
[ny,nx,nf,nb,ne,np,nalt] = dim_(m);

R = m.solution{2};

k0 = size(R,2)/ne - 1; % expansion up to t+k0 available
if k0 >= k, warning_(23,{k0}); return, end % expansion already available

R(:,end+(1:ne*(k-k0)),1:size(m.assign,3)) = NaN;
[flag,index] = isnan(m,'solution');
for ialt = find(~index)
  Xa = m.expand{1}(:,:,ialt);
  Xf = m.expand{2}(:,:,ialt);
  Ru = m.expand{3}(:,:,ialt);
  J = m.expand{4}(:,:,ialt);
  Jk = m.expand{5}(:,:,ialt);
  for i = k0+1 : k
    Ra = -Xa*Jk*Ru; % Jk stores J^(k-1)
    Jk = Jk*J;
    Rf = Xf*Jk*Ru;
    R(:,i*ne+(1:ne),ialt) = [Rf;Ra];
  end
  m.expand{5}(:,:,ialt) = Jk;
end

m.solution{2} = R;

end % of primary function -----------------------------------------------------------------------------------