function [M,list] = fmse(m,nper,varargin)
%
% FMSE  Forecast mean square error.
%
% Syntax: 
%   [M,list] = fmse(m,nper,...)
% Arguments:
%   M numeric, list cellstr, m model, nper numeric
% Options:
%   'select' cellstr|char|Inf
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'select',Inf,...
};
options = passopt(default,varargin{:});

if ischar(options.select), options.select = {options.select}; end

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
M = zeros([ny+nx,ny+nx,nper,nalt]);

[falg,index] = isnan(m,'solution');
for ialt = find(~index)
  [T,R,K,Z,H,D,Za,Omega] = sspace_(m,ialt);
  M(:,:,:,ialt) = fmse(T,R,K,Z,H,D,Za,Omega,nper);
end

% print variables
if nargout > 1 || iscellstr(options.select)
  ny = size(Z,1);
  nx = size(T,1);
  list = ref(printid_(m),1:ny+nx);
end

% select variables
if iscellstr(options.select)
  [M,index,list,notfound] = select(M,list,options.select);
  if ~isempty(notfound), warning_(1,notfound); end
end

end % of primary function -----------------------------------------------------------------------------------