function [W,list] = ifrf(m,freq,varargin);
%
% IFRF  Frequency response function to input signals.
%
% Syntax:
%   [W,list] = ifrf(m,freq,...)
% Arguments:
%   W cell, list cell, m model, freq numeric
% Options:
%   'select' char|cellstr|Inf (Inf)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'select',Inf,...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

freq = vech(freq);
nfreq = length(freq); 
[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
W = {zeros([ny+nx,ne,nfreq,nalt]),freq};

[flag,index] = isnan(m,'solution');
for ialt = find(~index)
  [T,R,K,Z,H,D,Za,Omega] = sspace_(m,ialt);
  W{1}(:,:,:,ialt) = ifrf(T,R,K,Z,H,D,Za,Omega,freq);
end

% print variable names
if nargout > 1 || iscellstr(options.select)
  tmp = printid_(m);
  list = {tmp(1:ny+nx),tmp(ny+nx+1:end)};
end

% select variables
if iscellstr(options.select)
  [W{1},index,list,notfound] = select(W{1},list,options.select);
  if ~isempty(notfound), warning_(1,notfound); end
end

end % of primary function -----------------------------------------------------------------------------------