function [P,mloglik,Grad,Hess,M,se2,varargout] = maxloglik(M,Data,Zd,D,varargin)
%
% MAXLOGLIK  Maximum likelihood estimation.
%
% Syntax:
%   [p,mloglik,Grad,Hess,m,se2] = maxloglik(m,data,Z,p,...)
% Arguments:
%   p dbase|struct; mloglik,Grad,Hess numeric; m model; se2 numeric; data cell; Z numeric
% Options:
%   see MODEL/LOGLIK options
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

[ny,nx,nf,nb,ne,np,nalt] = dim_(M);

% multiple parameterisations not allowed
if nalt > 1, error_(47,'MAXLOGLIK'); end 
[void,nper,ndata] = size(Data{1});
ixobserved = ~isnan(Data{1});

% list of optimised parameters
Name = vech(fieldnames(D));
Name(strmatch('IRIS_',Name)) = [];
index = isnan(findnames(M.name(M.nametype == 4),Name));
if any(index)
  warning_(31,Name(index));
  Name(index) = [];
  if isempty(Name), error_(53); end
end

% retrieve initial values and lower/upper bounds
p0 = []; pl = []; pu = [];
for i = 1 : length(Name)
  aux = D.(Name{i}); 
  p0(i) = aux(1);
  if isnan(p0(i)), p0(i) = M.assign(strcmp(M.name,Name{i})); end
  if length(aux) > 1, pl(i) = aux(2);
    else, pl(i) = -Inf; end
  if length(aux) > 2, pu(i) = aux(3);
    else, pu(i) = Inf; end
end  

% disregard parameters for which lower bound == upper bound
index = pl == pu;
if any(index)
  M = assign(M,Name(index),pl(index));
  Name(index) = [];
  p0(index) = [];
  pl(index) = [];
  pu(index) = [];
end
if all(isinf(pl)) && all(isinf(pu)), pl = []; pu = []; end

% maximise log-likelihood
[p,FVAL,EXITFLAG,OUTPUT,LAMBDA,Grad,Hess] = fmincon(@objfcn_,p0,[],[],[],[],pl,pu,[],optimset('display','iter','maxfunevals',5000,'GradObj','off','Hessian','off'));
M = solve(assign(M,Name,p),'persist',true);
[mloglik,se2,varargout{1:nargout-6}] = loglik(M,Data,Zd,{},varargin{:});
if se2 ~= 1
  % scale std errors
  M.assign(1,end-sum(M.nametype == 3)+1:end,1) = sqrt(se2)*M.assign(1,end-sum(M.nametype == 3)+1:end,1);
end
P = cell2struct(num2cell(p),Name,2);

% end of function body --------------------------------------------------------------------------------------

  function mloglik = objfcn_(p) % nested function -----------------------------------------------------------
  [M,npath] = solve(assign(M,Name,p),'persist',true);
  if npath ~= 1, mloglik = 1e20; return, end
   mloglik = loglik(M,Data,Zd,{},varargin{:});
  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------