function d = outside(m,d,range)
%
% OUTSIDE  Evaluate @outside equations.
%
% Syntax:
%   d = outside(m,d)
% Arguments:
%   d struct; m model
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if isempty(m.outside.lhs), return, end

% add steady-state level and growth
sslevel = get(m,'sslevel');
ssgrowth = get(m,'ssgrowth');
list = [get(m,'ynames'),get(m,'xnames')];
for i = 1 : length(list)
  d.(sprintf('sslevel_%s',list{i})) = tseries(range,sslevel.(list{i}));
  d.(sprintf('ssgrowth_%s',list{i})) = tseries(range,ssgrowth.(list{i}));
end

% add steady-state level and growth
p = get(m,'parameters');
list = fieldnames(p);
for i = 1 : length(list)
  d.(list{i}) = tseries(range,p.(list{i}));
end

for i = 1 : length(m.outside.lhs)
  if ~isfield(d,m.outside.lhs{i})
    d.(m.outside.lhs{i}) = tseries();
  end
  if ~isempty(m.outside.label{i})
    d.(m.outside.lhs{i}) = comment(d.(m.outside.lhs{i}),m.outside.label{i});
  end
end

for t = vech(range)
  rhs = strrep(m.outside.rhs,'(t,:)',sprintf('(%g,:)',t));
  for i = 1 : length(rhs)
    try
      d.(m.outside.lhs{i})(t,:) = eval(rhs{i},'NaN');
    catch, end
  end
end

end % of primary function -----------------------------------------------------------------------------------