function [m,deriv] = deriv_(m,eqselect,ialt)

persistent eqtn;
if nargin == 1, eqtn = cell([1,length(m.eqtnF)]); return, end

realsmall = getrealsmall();

deriv = m.deriv0;
assign = m.assign(1,:,ialt);
if any(eqselect), deriv1_(); end

if ialt == 1
  m.assign0(:) = m.assign(1,:,ialt);
  m.deriv0.c(:) = deriv.c;
  m.deriv0.f(:) = deriv.f;
end

  function deriv1_() % nested function ------------------------------------------------------------

  ny = m.nname(1);
  nx = length(m.systemid{2});
  ne = m.nname(3);
  np = m.nname(4);
  nt = size(m.occur,3);
  nvar = sum(m.nname(1:3));
  nderiv = nt*nvar;

  t = m.tzero;
  mint = 1 - t;
  maxt = nt - t;

  tvec = mint : maxt;
  initnon = trendarray_(m,1:sum(m.nname),tvec,false,ialt);
  initnon = shiftdim(initnon,-1);
  hnon = eps^(1/4)*max([initnon;ones([1,sum(m.nname),nt])],[],1);
  
  if m.linear == true
    initlin = zeros([1,sum(m.nname)]);
    initlin(1,m.nametype == 4) = real(assign(m.nametype == 4));
    initlin = initlin(:,:,ones([1,nt]));
  else
    initlin = initnon;
  end
  hlin = ones([1,sum(m.nname),nt]);

  plusnon = initnon + hnon;
  minusnon = initnon - hnon;
  stepnon = plusnon - minusnon;

  pluslin = initlin + hlin;
  minuslin = initlin - hlin;
  steplin = pluslin - minuslin;

  initnon(1,m.log,:) = exp(initnon(1,m.log,:));
  plusnon(1,m.log,:) = exp(plusnon(1,m.log,:));
  minusnon(1,m.log,:) = exp(minusnon(1,m.log,:));

  initlin(1,m.log,:) = exp(initlin(1,m.log,:));
  pluslin(1,m.log,:) = exp(pluslin(1,m.log,:));
  minuslin(1,m.log,:) = exp(minuslin(1,m.log,:));

  fgrid = cell([1,3]);
  fevaluate = cell([1,3]);

  for eq = find(eqselect)

    if m.nonlin(eq) == false
      init = initlin;
      plus = pluslin;
      minus = minuslin;
      step = steplin;
    else
      init = initnon;
      plus = plusnon;
      minus = minusnon;
      step = stepnon;
    end

    if isempty(eqtn) || length(eqtn) ~= length(m.eqtn) || isempty(eqtn{eq})
      eqtn{eq} = eval(['@(x,t) ',m.eqtnF{eq}]);
    end
  
    aux = permute(m.occur(eq,m.nametype <= 3,:),[3,2,1]);
    [tmocc,nmocc] = find(aux);
    n = length(nmocc);
    fgrid{2} = init;
    for i = [1,3]
      fgrid{i} = init(ones([1,n]),:,:);
    end
    for i = 1 : n
      fgrid{1}(i,nmocc(i),tmocc(i)) = minus(1,nmocc(i),tmocc(i));
      fgrid{3}(i,nmocc(i),tmocc(i)) = plus(1,nmocc(i),tmocc(i));      
    end
  
    for i = [1,3]
      x = fgrid{i};
      fevaluate{i} = eqtn{eq}(x,t);
    end
  
    if m.linear == true
      x = fgrid{2};
      deriv.c(eq) = eqtn{eq}(x,t);
    end
    
    index = [];
    value = [];
    for i = 1 : n
      index = [index,(tmocc(i)-1)*nvar+nmocc(i)];
      value = [value,(fevaluate{3}(i)-fevaluate{1}(i)) / step(1,nmocc(i),tmocc(i))];
    end
    deriv.f(eq,index) = value;

  end

  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------

%{
  % -----subfunction----- %
  
  function deriv = deriv2_(m,eqselect)

  [ixName,ixEqtn,nname,neqtn,sName,sEqtn] = index_(m);
  nt = size(m.occur,3);
  nvar = sum(m.nname(1:3));
  nderiv = nt*nvar;
  
  deriv = m.deriv;
  deriv.s = cell([1,sEqtn(3)]);

  t = m.tzero;
  
  if m.linear
    init = zeros([1,sName(4),nt]);
    init(1,m.nametype == 4,t) = real(assign(m.nametype == 4));
    step = ones([1,sName(4)]);
  else
    aux = real(m.assign);
    aux(m.log) = log(aux(m.log));
    init = repmat(aux,[1,1,nt]);
    step = eps^(1/4)*max([aux;ones([1,sName(4)])],[],1);
  end
  
  sGrid = cell([3,3]);
  sEval = cell([3,3]);
    
  for iEq = eqselect
  
    aux = permute((m.occur(iEq,[ixName{1:3}],:)),[3,2,1]);
    [tmocc,nmocc] = find(aux);
    n = length(nmocc);
    for i = [1,3]
      for j = [1,3]
        sGrid{i,j} = repmat(init,[n*(n+1)/2,1,1]);
      end
    end
  
    divide = zeros([n*(n+1)/2,1]);
    count = 1;
    for i = 1 : n
      for j = 1 : i
      	if i == j
      	  factor = 0.5;
      	else
      	  factor = 1;
      	end
      	for k = [1,3]
      	  for m = [1,3]
      	    sGrid{k,m}(count,nmocc(i),tmocc(i)) = sGrid{k,m}(count,nmocc(i),tmocc(i)) + (k-2)*step(nmocc(i))*factor;
      	    sGrid{k,m}(count,nmocc(j),tmocc(j)) = sGrid{k,m}(count,nmocc(j),tmocc(j)) + (m-2)*step(nmocc(j))*factor;
      	  end
      	end
      	divide(count) = step(nmocc(i))*step(nmocc(j))*factor*factor;
      	count = count + 1;
      end
    end
    
    for i = [1,3]
      for j = [1,3]
        x = sGrid{i,j};
        sEval{i,j} = eval(m.eqtnF{iEq});
      end
    end
    
    aux = (sEval{3,3}+sEval{1,1}-sEval{1,3}-sEval{3,1})./(4*divide);
  
    tmp = sparse(nt*sName(3),nt*sName(3));
    count = 1;
    for i = 1 : n,
      for j = 1 : i
      	tmp((tmocc(i)-1)*sName(3)+nmocc(i),(tmocc(j)-1)*sName(3)+nmocc(j)) = aux(count);
      	tmp((tmocc(j)-1)*sName(3)+nmocc(j),(tmocc(i)-1)*sName(3)+nmocc(i)) = aux(count);
      	count = count + 1;
      end
    end
    deriv.s{iEq} = tmp;

  end

  end
%}