function s = sprintf(m,varargin)
%
% SPRINTF  Write formatted model data to string.
%
% Syntax:
%   s = sprintf(m,...)
% Arguments:
%   s char; m model
% Options:
%   'dbname' char ('p')
%   'log' cellstr|char (empty)
%   'format' char ('%.16g')
%   'what' char ('parameters')
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'dbname','p',...
  'what','parameters',...
  'format','%.16g',...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

if size(m.assign,3) > 1, error_(47,{'SPRINTF|FPRINFT'}); end

s = '';
switch options.what
case {'p','parameter','parameters'}
  format = '%s.%s = #; %% %s\n';
  format = strrep(format,'#',options.format);
  for i = find(m.nametype == 4)
    s = [s,sprintf(format,options.dbname,m.name{i},m.assign(i),m.namelabel{i})];
  end
case {'std'}
  ne = sum(m.nametype == 3);
  format = '%s.%s = #; %% %s\n';
  format = strrep(format,'#',options.format);
  index = find(m.nametype == 4);
  for i = index(end-ne+1:end)
    s = [s,sprintf(format,options.dbname,m.name{i},m.assign(i),m.namelabel{i})];
  end
end

end % of primary function -----------------------------------------------------------------------------------