function [Phi,Phicum,list] = srf(m,nper,varargin)
%
% SRF  Shock (impulse) response function (VMA representation).
%
% Syntax:
%   [Phi,Phicum,list] = srf(m,nper,...)
% Arguments:
%   Phi,Phicum numeric; list cellstr; m model; nper numeric
% Options:
%   'select' cellstr|char|Inf (Inf)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'select',Inf,...
};
options = passopt(default,varargin{:});

if ischar(options.select), options.select = {options.select}; end

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
Phi = zeros([ny+nx,ne,nper,nalt]);
[flag,index] = isnan(m,'solution');
for ialt = find(~index)
  [T,R,K,Z,H,D,Za,Omega] = sspace_(m,ialt);
  Phi(:,:,:,ialt) = srf(T,R,K,Z,H,D,Za,Omega,nper);
end
if nargout > 1, Phicum = cumsum(Phi,3); end % cumulative response function

% print variables
if nargout > 2 || iscellstr(options.select)
  tmp = printid_(m);
  list = {tmp(1:ny+nx),tmp(ny+nx+1:end)};
end

% select variables
if iscellstr(options.select)
  [Phi,index,list,notfound] = select(Phi,list,options.select);
  if nargout > 1, Phicum = Phicum(index{1},index{2},:,:); end
  if ~isempty(notfound), warning_(1,notfound); end
end

end % of primary function -----------------------------------------------------------------------------------