function d = sstatedb(m,range,varargin)
%
% SSTATEDB  Model-specific steady-state source database.
%
% Syntax:
%   d = sstatedb(m,range,...)
% Arguments:
%   d struct, m model, range numeric
% Options:
%   'alt' numeric|logical|Inf (Inf)
%   'ndraw' numeric (1)
%   'precision' char (get(m,'precision'))
%   'residuals' function_handle|char (@zeros)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'alt',inf,...
  'ndraw',1,...
  'precision',m.precision,...
  'residuals','',...
};
options = passopt(default,varargin{:});

if ~isnumeric(range), error('Incorrect type of input argument(s).'); end
if ischar(options.residuals) && ~isempty(options.residuals), options.residuals = eval(sprintf('@(dim) %s(dim)',options.residuals)); end

if any(isinf(options.alt)), options.alt = 1 : size(m.assign,3);
  elseif islogical(options.alt), options.alt = find(options.alt); end

% function body ---------------------------------------------------------------------------------------------

range = range(1) : range(end);
ne = sum(m.nametype == 3);
nper = length(range);
id = vech(find(m.nametype <= 3));
tvec = double(round(range - range(1)));

if options.ndraw > 1 && length(options.alt) > 1
  error('Multiple draws only permitted for a single parameterisation.');
end

d = struct();
% create steady state array
X = trendarray_(m,id,tvec,true,options.alt,options.precision);
if options.ndraw > 1, X = X(:,:,ones([1,options.ndraw])); end
% convert steady state array to time series
for i = id, d.(m.name{i}) = tseries(range,permute(X(i,:,:),[2,3,1]),m.namelabel{i}); end

% generate random residuals if requested
if ~isempty(options.residuals) && ~strcmp(char(options.residuals),'zeros')
  d = addrand_(m,d,options.residuals,[nper,max([length(options.alt),options.ndraw]),ne]);
end

% add parameters database
d = addpardb_(m,d);

end % of primary function -----------------------------------------------------------------------------------