function [S,D,list] = xsf(m,freq,varargin)
%
% XSF  Power spectrum and spectral density function.
%
% Syntax:
%   [S,D,list] = xsf(m,freq,...)
% Arguments:
%   S,D cell; list cellstr; m model; freq numeric
% Options:
%   'select' cellstr|char|Inf (Inf)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'select',Inf,...
};
options = passopt(default,varargin{:});

if ischar(options.select), options.select = {options.select};
  elseif isempty(options.select), options.select = {''}; end

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

freq = vech(freq);
nfreq = length(freq);
[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
S{1} = zeros([ny+nx,ny+nx,nfreq,nalt]);
S{2} = freq;

[flag,index] = isnan(m,'solution');
for ialt = find(~index)
  [T,R,K,Z,H,D,Za,Omega] = sspace_(m,ialt);
  S{1}(:,:,:,ialt) = xsf(T,R,K,Z,H,D,Za,Omega,freq);
end

% power spectrum to spectral density conversion
if nargout > 1
  C = acf(m);
  D = {xsf2sdf(S{1},C),S{2}};
end

% list of variables
if nargout > 2 || iscellstr(options.select)
  ny = size(Z,1);
  nx = size(T,1);
  list = ref(printid_(m),1:ny+nx);
end

% select variables
if iscellstr(options.select)
  [S{1},index,list,notfound] = select(S{1},list,options.select);
  if ~isempty(notfound), warning_(1,notfound); end
  if nargout > 1, D{1} = D{1}(index,index,:,:); end
end

end % of primary function -----------------------------------------------------------------------------------