function dp = zerodb(m,range,varargin)
%
% ZERODP  Zero source datapack for model. 
%
% dp = zerodb(m,range,...)
% Arguments:
%   dp cell, m model, range numeric
% Options:
%   'ndraw' numeric (1)
%   'residuals' function_handle|char (@zeros)
%   'precision' char ('double')
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'ndraw',1,...
  'residuals','',...
  'precision','double',...
};
options = passopt(default,varargin{:});

if ~isnumeric(range), error('Incorrect type of input argument(s).'); end
if ischar(options.residuals) && ~isempty(options.residuals), options.residuals = eval(sprintf('@(dim) %s(dim)',options.residuals)); end

% function body ---------------------------------------------------------------------------------------------

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
range = single(range(1) : range(end));
nper = length(range);

if options.ndraw > 1 && nalt > 1, error('Multiple draws only permitted for single parameterization.'); end
ndata = max([options.ndraw,nalt]);

dp = {zeros([ny,nper,ndata],options.precision),zeros([nx,nper,ndata],options.precision),zeros([ne,nper,ndata],options.precision),range,meta(m)};
if m.linear == false
  id = m.solutionid{1};
  dp{1}(m.log(real(id)),:,:) = 1;
  id = m.solutionid{2};
  dp{2}(m.log(real(id)),:,:) = 1;
end

% generate random residuals if requested
if ~isempty(options.residuals) && ~strcmp(char(options.residuals),'zeros')
  dp{3}(:,:,:) = options.residuals([ne,nper,ndata]);
  stdvec = m.assign(1,end-ne+1:end,:);
  if ndata > 1 && nalt == 1, stdvec = stdvec(:,:,ones([1,ndata])); end
  for ialt = 1 : ndata
    x = transpose(stdvec(1,:,ialt));
    dp{3}(:,:,ialt) = x(:,ones([1,nper])).*dp{3}(:,:,ialt);
  end
end

end % of primary function -----------------------------------------------------------------------------------