function p = endogenize(p,name,range)
%
% ENDOGENIZE  Endogenize residuals and/or previously exogenized variables.
%
% Syntax:
%   p = endogenize(p,name,range)
% Arguments:
%   p plan; name cellstr|char; range numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if (~iscellstr(name) && ~ischar(name)) || ~isnumeric(range), error('Incorrect type of input argument(s).'); end
if ischar(name), name = {name}; end

% function body ---------------------------------------------------------------------------------------------

if ~isnan(p.range)
  flag = false;
  before = round(range - p.range(1)) < 0;
  after = round(range - p.range(end)) > 0;
  tmp = [range(before),range(after)];
  if ~isempty(tmp), warning_(6,format_(name,tmp)); end
end

if ~isempty(range) && ~isempty(name)
  modelname = p.meta.name(real(p.meta.id{3}));
  [p.endogenized,p.exogenized,unable] = datapoints_(modelname,p.endogenized,p.exogenized,name,range);
  if ~isempty(unable), warning_(2,format_(unable,range)); end
end

end % of primary function -----------------------------------------------------------------------------------

  function x = format_(name,range) % subfunction ------------------------------------------------------------

  tmp = dat2str(range);
  tmp = sprintf(' %s',tmp{:});
  x = {};
  for i = 1 : length(name), x(end+1) = {sprintf('''%s'' in period(s)%s',name{i},tmp)}; end

  end % of subfunction --------------------------------------------------------------------------------------