function w = backward(w)
%
% BACKWARD  Reverse VAR process.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if ~isempty(w.B), error_(12); end

[ny,p,nalt] = dim(w);
[A,B,K,Omega] = companion(w);
C = acf(w,'order',p-1);

for i = 1 : nalt 
  % Sigma = [C0,C1,C2;C1',C0,C1;C2',C1',C0]
  X = C(:,:,:,ialt);
  Sigma = reshape(X,[ny,ny*p]);
  for i = 2 : p
    X(:,:,2:end) = X(:,:,1:end-1);
    X(:,:,1) = transpose(C(:,:,i));
    Sigma = [Sigma;reshape(X,[ny,ny*p])];
  end
  % projection matrix
  P = Sigma*transpose(A) / Sigma;
  % covariance matrix of residuals
  Omegab = Sigma - P*Sigma*transpose(P);
  Omegab = Omegab(end-ny+1:end,end-ny+1:end);
  P = P(end-ny+1:end,:);
  P = reshape(P,[ny,ny,p]);
  P = P(:,:,end:-1:1);
  % transition matrix
  Ab = reshape(P,[ny,ny*p]);
  % implied constant vector
  Kb = sum(var2poly(Ab),3)*mean(w);

  w.A(:,:,ialt) = Ab;
  w.K(:,1,ialt)) = Kb;
  w.Omega(:,:,ialt) = Omegab;
end

end % of primary function -----------------------------------------------------------------------------------