function varargout = get(w,varargin)
%
% GET  Get VAR attributes.
%
% Syntax:
%   [<value>] = get(w,<attrib>)
% Arguments:
%   value: ?; w: rvar|svar
% Attributes:
%   'A' returns numeric
%   'B' returns numeric
%   'K' returns numeric
%   'Omega' returns numeric
%   'resid' returns numeric
%   'sample' returns numeric
%   'AIC' returns numeric
%   'SBC' returns numeric
%   'eig' returns numeric
%   'nhyper' returns numeric
%   'comment' returns char
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if length(varargin) ~= nargout && length(varargin) ~= 0 && nargout ~= 0, error('Incorrect number of input or output arguments.'); end
if ~iscellstr(varargin), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);
varargout = {};
unrecognized = {};

for i = 1 : length(varargin)
  switch strtrim(lower(varargin{i}))
  case 'a'
    if all(size(w.A) == 0), x = [];
      else, x = var2poly(w.A); end
  case 'b'
    if isempty(w.B)
      x = eye(ny);
      x = x(:,:,ones([1,nalt]));
    else
      x = w.B;
    end
  case {'const','c','k'}, x = w.K;
  case 'omega'
    if isempty(w.B) % rvar
      x = w.Omega;
    else % svar
      x = zeros([ny,ny,nalt]);
      for ialt = 1 : nalt
        x(:,:,ialt) = diag(ones([1,ny])*w.std(ialt)*w.std(ialt));
      end
    end
  case 'aic', x = w.aic;
  case 'sbc', x = w.sbc;
  case {'eig','eigval'}, x = eig(w);
  case 'nhyper', x = w.nhyper;
  case 'nper', x = length(w.sample);
  case 'resid', x = w.resid;
  case 'sample', x = w.sample;
  case 'stdvec'
    if isempty(w.B), x = vech(sqrt(diag(w.Omega)));
      else, x = ones([1,ny])*w.std; end
  case 'comment', x = w.comment;
  otherwise
    unrecognized{end+1} = varargin{i};
    x = [];
  end
  varargout{end+1} = x;
end

if ~isempty(unrecognized), error('Unrecognised attribute: ''%s''.\n',unrecognized{:}); end

end % of primary function -----------------------------------------------------------------------------------