function varargout = kstep(var,dpack,varargin)
%
% KSTEP  K-step-ahead VAR forecasts and errors.
%
% [x1,...] = kstep(w,dpack,...)
% Arguments:
%   x1 cell, w rvar|svar, dpack cell
% Options:
%   'deviation' logical (false)
%   'horizon' numeric (Inf)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'deviation',false,...
  'horizon',Inf,...
  'steps',1,...
};
options = passopt(default,varargin{1:end});

% -----function RVAR.KSTEP body----- %

[ny,order,nZ] = dim(var);
nper = size(dpack{1},2);

if isinf(options.horizon)
  horizon = nper - order;
else
  horizon = round(options.horizon);
end
if horizon <= 0
  error_(7);
end

dpack{3}(isnan(dpack{3})) = 0;
if size(dpack{3},2) < nper
  dpack{3} = [dpack{3},zeros([ny,nper-size(dpack{3},2)])];
end

[A,B,K,Omega,dpack] = companion(var,dpack);
if options.deviation == true
  K(:) = 0;
end
fcast = cell([1,1+options.steps]);
[fcast{:}] = deal(cell([1,4]));
fcast{1} = dpack;
for k = 1 : options.steps
  fcast{1+k} = dpack;
  for t = nper-horizon+1 : nper
    fcast{1+k}{1}(:,t) = K + A*fcast{1+k-1}{1}(:,t-1);
  end
  fcast{1+k}{3} = dpack{1} - fcast{1+k}{1};
  fcast{1+k}{3}(:,1:nper-horizon) = NaN;
end

for i = 1 : options.steps
  varargout{i} = fcast{1+i};
  varargout{i}{1} = varargout{i}{1}(1:ny,:);
  varargout{i}{3} = varargout{i}{3}(1:ny,:);
end

end