function [stat,crit] = lrtest(wu,wr,level)
%
% LRTEST  Likelihood ratio test of two VARs.
%
% Syntax:
%   [stat,crit] = lrtest(w1,w2,level)
% Arguments:
%   stat numeric; crit numeric; w1,w2 rvar|svar; level numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(wu);
if size(wu.A,3) > 1 || size(wr.A,3) > 1, error_(11,{'LRTEST'}); end
if ~rngcmp(wu.sample,wr.sample), error_(2); end
if wu.nhyper == wr.nhyper, warning_(3); end

% swap restricted and unrestricted if needed
if wu.nhyper < wr.nhyper
  aux = wu;
  wu = wr;
  wr = aux;
end

nper = get(wu,'nper');
stat = zeros([1,nalt]);
for ialt = 1 : nalt
  if ialt == 1 || size(wr.Omega,3) > 1, Omegar = wr.Omega(:,:,ialt); end
  if ialt == 1 || size(wu.Omega,3) > 1, Omegau = wu.Omega(:,:,ialt); end
  stat(ialt) = nper*(log(det(Omegar)) - log(det(Omegau)));
end
crit = chi2inv(1-level,wu.nhyper-wr.nhyper);

end % of primary function -----------------------------------------------------------------------------------