function [stat,crit] = portest(w,varargin)
%
% PORTEST  Portmanteau test for autocorrelation of VAR residuals.
%
% Syntax:
%   [stat,crit] = portest(w,horizon,level)
% Arguments:
%   stat numeric; crit numeric; w rvar|svar; horizon numeric; level numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

[ny,p,nalt] = dim(w);

% old syntax portest(w,data,horizon,level)
if nargin > 1 && iscell(varargin{1})
  varargin(1) = [];
  warning('This is an obsolete syntax PORTEST syntax.');
end

if length(varargin) > 0, horizon = varargin{1};
  else, horizon  = p + 1; end
if length(varargin) > 1, level = varargin{2};
  else, level = 0.05; end

% function body ---------------------------------------------------------------------------------------------

if horizon <= p, error_(8); end

% reduced-form residuals
resid = w.resid;

B = w.B;
% default rvar to svar conversion
if isempty(B), B = transpose(chol(w.Omega)); end
resid = B\resid;

nper = size(resid,2);
stat = zeros([1,nalt]);
stat1 = zeros([1,nalt]);
for ialt = 1 : nalt
  for i = 1 : horizon
    C0i = resid(:,1+i:end,ialt)*transpose(resid(:,1:end-i,ialt)) / nper;
    stat(ialt) = stat(ialt) + trace(transpose(C0i)*C0i) / (nper-i);
  end
end
stat = nper^2*stat;
if nargout > 1, crit = chi2inv(1-level,ny^2*(horizon-p)); end

end % of primary function -----------------------------------------------------------------------------------