function w = set(w,varargin)
%
% SET  Set VAR attributes.
%
% Syntax:
%   w = set(w,<attrib,value>)
% Arguments:
%   w rvar|svar
% Attributes:
%   'A' numeric
%   'B' numeric
%   'K' numeric
%   'comment' char
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~iscellstr(varargin(1:2:end)), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

realsmall = getrealsmall();
[ny,p,nalt] = dim(w);
varargout = {};
unrecognized = {};

for i = 1 : 2 : length(varargin)
  switch strtrim(lower(varargin{i}))
  case 'a', w.A(:) = varargin{i+1}(:); w = eig(w);
  case 'b'
    w.B = varargin{i+1}(:);
    if nalt > 1 && size(w.B,3) == 1, w.B = w.B(:,:,ones([1,nalt])); end
  case {'const','c','k'}, w.K(:) = varargin{i+1}(:);
  case 'comment', w.comment = varargin{i+1};
  case 'mean'
    nonstat = [];
    eigval = eig(w);
    mean_ = mean(w);
    x = varargin{i+1};
    if nalt > 1 && size(x,3) == 1, x = x(:,1,ones([1,nalt])); end
    for ialt = 1 : nalt
      index = isnan(x(:,1,ialt));
      if any(index), x(index,1,ialt) = mean_(index,1,ialt); end
      if any(abs(eigval(1,:,ialt)) > 1-realsmall), nonstat(end+1) = ialt;
        else, w.K(:,1,ialt) = sum(var2poly(w.A(:,:,ialt)),3)*x(:,1,ialt); end
    end
    if ~isempty(nonstat), warning_(7,{sprintf(' %g',nonstat)}); end
  otherwise
    unrecognized{end+1} = varargin{i};
  end
end % of for i = 1 : nargin-1

if ~isempty(unrecognized), error('Unrecognised attribute: ''%s''.\n',unrecognized{:}); end

end % of primary function -----------------------------------------------------------------------------------