function x = bpass(x,range,band,varargin)
%
% BPASS  Christiano-Fitzgerald band-pass filter.
%
% Syntax:
%   x = bpass(x,range,band,...)
% Arguments:
%   x tseries; range numeric; band numeric
% Options:
%   'log' logical (false)
%   'detrend' logical (true)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~isnumeric(band) || ~isnumeric(range), error('Incorrect type of input argument(s).'); end
if length(band) ~= 2 && length(range) == 2, [range,band] = deal(band,range); end
range = setrange(range,x.start:x.start+size(x.data,1)-1);

% function body ---------------------------------------------------------------------------------------------

if isempty(range)
  [x.data,dim] = reshape_(x.data);
  x.data = x.data([],:);
  x.data = reshape_(x.data,dim);
  x.start = NaN;
  return
end

data = getdata_(x,range);
[data,dim] = reshape_(data);
data = bpass(data,band,varargin{:});
x.data = reshape_(data,dim);
x.start = range(1);
x = cut_(x);

end % of primary function -----------------------------------------------------------------------------------