function [data,dates,cmt] = double(x,dates)
%
% DOUBLE  Convert time series to numeric array.
%
% Syntax:
%   [x,dates,cmt] = double(x)
%   [x,dates,cmt] = double(x,userdates)
% Arguments:
%   x numeric, dates numeric, cmt cellstr, x tseries, userdates numeric|char
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if nargin < 2, dates = 'max'; end
if isnumeric(dates) && any(isinf(dates)), dates = 'min'; end

if nargin > 1 && ~isa(dates,'char') && ~isa(dates,'function_handle') && ~isnumeric(dates)
  error('Incorrect type of input argument(s).');
end

% function body ---------------------------------------------------------------------------------------------

if ischar(dates)
  switch dates
  case {'min','i','intersection'}
    [data,dim,nper] = reshape_(x.data);
    dates = x.start : x.start + nper - 1;
    index = any(isnan(data),2);
    data(index,:) = [];
    dates(index) = [];
    data = reshape_(data,dim);
  case {'max','u','union'}
    dates = x.start : x.start + size(x.data,1) - 1;
    data = x.data;
  end
else
  dates = vech(dates);
  data = getdata_(x,dates);
end

if nargout > 2, cmt = x.comment; end

end % of primary function -----------------------------------------------------------------------------------