function [handle,range] = graph_(fcn,factor,varargin)
%
% TSERIES/PRIVATE/GRAPH_
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if length(varargin) > 2 && isnumeric(varargin{1}) && isnumeric(varargin{2})
  ax = varargin{1};
  range = varargin{2};
  varargin(1:2) = [];
  setxlim = true;
elseif length(varargin) > 1 && isnumeric(varargin{1})
  ax = gca();
  range = varargin{1};
  varargin(1) = [];
  setxlim = true;
else 
  ax = gca();
  range = Inf;
  setxlim = false;
end

x = varargin{1};
varargin(1) = [];

flag = true;
try
  if isempty(varargin)
    plotspec = {};
    options.dateformat = '';
    options.datetick = Inf;
    options.function = [];
    syntax = 'new';
  elseif length(varargin) == 1
    [plotspec,options] = oldargin_(varargin{1});
    syntax = 'old';
  elseif cellfun(@ischar,varargin(1:2:end)) | cellfun(@iscellstr,varargin(1:2:end))
    [plotspec,options] = newargin_(varargin{:});
    syntax = 'new';
  else
    flag = false;
  end
catch
  if cellfun('isclass',varargin(1:2:end),'char') | cellfun('isclass',varargin(1:2:end),'cell')
    [plotspec,options] = newargin_(varargin{:});
  else
    flag = false;
  end
end
if flag == false, error('Incorrect type of input argument(s).'); end

if isempty(options.dateformat)
  irisconfig = load('irisconfig.mat');
  options.dateformat = irisconfig.plotdateformat;
end

% function body ---------------------------------------------------------------------------------------------

handle = [];
[data,range] = getdata_(x,range);
[data,dim] = reshape_(data);

if isempty(range)
  handle = fcn([0,1],NaN*ones([2,length(x)]));
  set(ax,'xlim',[0,1],'xtick',[0 1],'ylim',[0 1],'ytick',[0 1]);
  set(ax,'xticklabel',strvcat('NaN','NaN'),'yticklabel',strvcat('NaN','NaN'));
  return
end

[year,per,freq] = dat2ypf(range);
freq = freq(1);

% check for existing line plots when nextplot is add
isline = false;
add = false;
if strcmp(get(ax,'nextplot'),'add')
  add = true;
  h = get(ax,'children');
  for i = vech(h)
    isline = isline | strcmp(get(i,'type'),'line');
  end
end

% unable to add tseries line(s) to existing non-tseries lines
if isline == true
  tag = get(ax,'tag');
  if ~strcmp(tag,'tseries'), error('Current figure contains non-tseries line(s). Unable to add tseries lines.'); end
  set(ax,'xticklabelmode','auto');
end

set(ax,'xtickmode','auto','xticklabelmode','auto');
time = dat2dec(range);
if strcmp(syntax,'old')
  handle = fcn(ax,time,data,plotspec{:});
else
  handle = fcn(ax,time,data);
  if ~isempty(plotspec), set(handle,plotspec{:}); end
end

% first tick = start of first year
firsttick = floor(time(1));
% last tick = end of last year
lasttick = ceil(time(end));
if any(freq == [2,4,12]), lasttick = lasttick - 1/freq; end
% fix numerical inaccuracy
if lasttick < time(end), lasttick = time(end); end
set(gca,'xlim',[firsttick,lasttick]);

setdatetick(ax,freq,options.dateformat,options.datetick);
% adjust xlim if user's plot range specified
if ~isnan(freq) && setxlim == true && length(time) > 1
  factor = factor/iff(freq==0,1,freq);
  set(ax,'xlim',[time(1)-factor,time(end)+factor],'xlimmode','manual');
end

set(ax,'tag','tseries','userdata',freq);

% perform user's function
if ~isempty(options.function), options.function(handle); end

end % of primary function -----------------------------------------------------------------------------------

  function [plotspec,options] = oldargin_(plotspec) % subfunction -------------------------------------------

  options.dateformat = '';
  options.datetick = Inf;
  options.function = [];

  if ischar(plotspec)
    i = find(plotspec == '|',1);
    if ~isempty(i)
      options.dateformat = strtrim(plotspec(i+1:end));
      plotspec = {plotspec(1:i-1)};
    else
      plotspec = {plotspec};
    end
  elseif isa(plotspec,'function_handle')
    options.function = plotspec;
    options.dateformat = strtrim(plotspec(Inf));
    plotspec = {''};
  end

  end % of subfunction --------------------------------------------------------------------------------------

  function [plotspec,options] = newargin_(varargin) % subfunction -------------------------------------------

  [plotspec{1:length(varargin)}] = deal(varargin{:});
  plotspec(1:2:end) = strtrim(plotspec(1:2:end));

  list = {'dateformat','datetick','function'};
  default = {'',Inf,[]};
  for i = 1 : length(list)
    index = strcmpi(plotspec,list{i});
    if any(index)
      aux = find(index);
      options.(list{i}) = plotspec{aux(end)+1};
      plotspec([aux,aux+1]) = [];
    else
      options.(list{i}) = default{i};
    end
  end

  end % of subfunction --------------------------------------------------------------------------------------