function dpack = db2dpack(me,d,range,type,takelog,initcond)
%
% DB2DPACK  Database to datapack conversion.
%
% Syntax:
%   dpack = db2dpack(me,d,range)
% Arguments:
%   me struct; d struct; range numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~isstruct(d) || ~isnumeric(range)
  error('Incorrect type of input argument(s).');
end

if nargin < 4, type = 1 : 3; end
if nargin < 5, takelog = true; end
if nargin < 6, initcond = true; end

% function body ---------------------------------------------------------------------------------------------

ny = length(me.id{1});
nx = length(me.id{2});
ne = length(me.id{3});
nper = length(range);
nalt = NaN;

notfound = {};
invalid = {};

if any(type == 2)
  [dpack{2},notfound_,invalid_] = db2mat(d,range,me.name(real(me.id{2})),imag(me.id{2}),initcond);
  if takelog == true, dpack{2}(me.log(real(me.id{2})),:,:) = log(dpack{2}(me.log(real(me.id{2})),:,:)); end
  notfound = [notfound,notfound_];
  invalid = [invalid,invalid_];
  nalt = size(dpack{2},3);
else
  dpack{2} = nan(0,me.precision);
end

if any(type == 1) && ny > 0
  [dpack{1},notfound_,invalid_] = db2mat(d,range,me.name(real(me.id{1})),imag(me.id{1}),initcond);
  if takelog == true, dpack{1}(me.log(real(me.id{1})),:,:) = log(dpack{1}(me.log(real(me.id{1})),:,:)); end
  notfound = [notfound,notfound_];
  invalid = [invalid,invalid_];
else
  if isnan(nalt), dpack{1} = nan(0,me.precision);
    else dpack{1} = nan([ny,nper,nalt],me.precision); end
end

if any(type == 3) && ne > 0
  [dpack{3},notfound_,invalid_] = db2mat(d,range,me.name(real(me.id{3})),imag(me.id{3}),initcond);
  notfound = [notfound,notfound_];
  invalid = [invalid,invalid_];
else
  if isnan(nalt), dpack{3} = nan(0,me.precision);
    else dpack{3} = nan([ne,nper,nalt],me.precision); end
end

if ~isempty(invalid), error_(4,invalid); end

dpack{4} = range;
dpack{5} = me;

end % of primary function -----------------------------------------------------------------------------------