function d = dbmerge(varargin)
%
% DBMERGE  Merge two or more databases field by field.
%
% Syntax:
%   d = dbmerge(<d>)
%   e = dbmerge(e,names,values)
% Arguments:
%   d dbase|struct; names cellstr; values cell
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~all(cellfun(@isstruct,varargin)) && (nargin ~= 3 || ~isstruct(varargin{1}) || ~iscellstr(varargin{2}) || ~iscell(varargin{3}))
  error('Incorrect type of input argument(s).');
end

% function body ---------------------------------------------------------------------------------------------

if nargin == 0
  d = struct();
  return
elseif nargin == 1
  d = varargin{1};
  return
end

names = vech(fieldnames(varargin{1}));
values = vech(struct2cell(varargin{1}));

if iscellstr(varargin{2})
  names = [vech(fieldnames(varargin{1})),vech(varargin{2})];
  values = [vech(struct2cell(varargin{1})),vech(varargin{3})];
else
  for i = 2 : nargin
    names = [names,vech(fieldnames(varargin{i}))];
    values = [values,vech(struct2cell(varargin{i}))];
  end
end
[names,index] = unique(names);

d = cell2struct(values(index),names,2);

end % of primary function -----------------------------------------------------------------------------------