function [rad,per] = xs2shift(xs,varargin)
%
% XS2SHIFT  Compute phase shift (radians and periods of time) from cross-spectrum.
%
% Syntax:
%   [rad,per] = xs2shift(xs,...)
% Arguments:
%   rad numeric, per numeric, xs cell
% Options:
%   'continuous' logical (false) If true remove discontinuities in phase shift vectors.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'continuous',false,...
  'threshold',3.5
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

nfreq = length(xs{2});

warning('off','MATLAB:divideByZero');
  rad = atan2(imag(xs{1}),real(xs{1}));
warning('on','MATLAB:divideByZero');
if options.continuous == true
  for i = 1 : size(xs{1},1)
    for j = 1 : size(xs{1},2)
      d = rad(i,j,1:end-1) - rad(i,j,2:end);
      index = find(abs(vech(d)) > options.threshold);
      for k = index
        rad(i,j,k+1:end) = rad(i,j,k+1:end)+2*pi*sign(d(k));
      end
    end
  end
end

%for i = 1 : min([size(xs{1},1),size(xs{1},2)])
%  rad(i,i,:) = 0;
%end

rad = {rad,xs{2}};
if nargout == 1, return, end

realsmall = getrealsmall;
per = rad;
for i = 1 : length(xs{2})
  if abs(xs{2}(i)) < realsmall
    per(:,:,i) = NaN;
  else
    per(:,:,i) = per(:,:,i) / xs{2}(i);
  end
end
per = {per,xs{2}};

end % of primary function -----------------------------------------------------------------------------------