function S = xsf(T,R,K,Z,H,D,U,Omega,freq,filter,applyto,order)
%
% FREQ-DOMAIN/XSF  Power spectrum function for state-space representations.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

isfilter = nargin > 9;

ny = size(Z,1);
[nx,nb] = size(T);
nf = nx - nb;
ne = size(R,2);
realsmall = getrealsmall();

Tf = T(1:nf,:);
Rf = R(1:nf,:);
Ta = T(nf+1:end,:);
Ra = R(nf+1:end,:);
Sigmaaa = Ra*Omega*transpose(Ra);
Sigmaff = Rf*Omega*transpose(Rf);
Sigmayy = H*Omega*transpose(H);
Sigmayf = Z*Ra*Omega*transpose(Rf);
Sigmafa = Rf*Omega*transpose(Ra);

T = [zeros([nf+nb,nf]),T];

freq = vech(freq);
nfreq = length(freq);

if isfilter == true, n = 1+order;
  else n = nfreq; end
S = zeros([ny+nf+nb,ny+nf+nb,n]);
s = zeros([ny+nf+nb,ny+nf+nb,1]);
warning('off');
Z0 = Z;
Z0(abs(Z) <= realsmall) = 0;
U0 = U;
U0(abs(U0) < realsmall) = 0;
for i = 1 : nfreq
%  F = eye(nf+nb) - T*exp(-1i*freq(i));
%  s(ny+1:end,ny+1:end) = F \ Sigmax / ctranspose(F);
  s(ny+1:end,ny+1:end) = inverse_();
  s(1:ny,1:ny) = Z0*s(ny+nf+1:end,ny+nf+1:end)*transpose(Z0) + Sigmayy;
  s(1:ny,ny+1:end) = Z0*s(ny+nf+1:end,ny+1:end);  
  s(ny+1:end,1:ny) = s(ny+1:end,ny+nf+1:end)*transpose(Z0);
  if ~isempty(U0)
    s(ny+nf+1:end,:) = U0*s(ny+nf+1:end,:);
    s(:,ny+nf+1:end) = s(:,ny+nf+1:end)*transpose(U0);
  end
  if isfilter == true
    s(applyto,:) = filter(i)*s(applyto,:);            
    s(:,applyto) = s(:,applyto)*conj(filter(i));            
    S(:,:,1) = S(:,:,1) + s;
    for j = 1 : order, S(:,:,1+j) = S(:,:,1+j) + s*exp(1i*freq(i)*j); end
  else
    S(:,:,i) = s;
  end
end
warning('on');

if isfilter == false
  % if isfilter == true we should divide S by 2*pi here but then multiply it by 2*pi afterwards in ACF: we therefore skip it in both steps
  S = S/(2*pi);
  for i = 1 : ny+nx, S(i,i,:) = real(S(i,i,:)); end
end

  function [Sxx,Saa] = inverse_() % nested function ---------------------------------------------------------
  A = Tf*exp(-1i*freq(i));
  B = eye(nb) - Ta*exp(-1i*freq(i));
  Saa = B\Sigmaaa/ctranspose(B);
  Sfa = Sigmafa/ctranspose(B) + A*Saa;
  X = A/B*transpose(Sigmafa);
  Sff = Sigmaff + X + ctranspose(X) + Tf*Saa*transpose(Tf);
  Sxx = [Sff,Sfa;ctranspose(Sfa),Saa];
  Sxx(isnan(Sxx)) = Inf;
  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------