function [C,diffuse] = acovf(T,R,K,Z,H,D,Za,Omega,eigval,order)
%
% TIME-DOMAIN/ACOVF  General state-space autocovariance function.
% 
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

ny = size(Z,1);
[nx,nb] = size(T);
nf = nx - nb;
ne = size(R,2);

if ny == 0
  Z = zeros([0,nb]);
  H = zeros([0,ne]);
  D = zeros([0,1]);
end

Tf = T(1:nf,:);
Ta = T(nf+1:end,:);
Rf = R(1:nf,:);
Ra = R(nf+1:end,:);

[dy,df,da] = isdiffuse(eigval,Z,Tf,eye(nb));
if ~isempty(Za), db = isdiffuse(eigval,Za);
  else db = da; end
Caa = zeros(nb);
Caa(~da,~da) = lyapunov(Ta(~da,~da),Ra(~da,:)*Omega*transpose(Ra(~da,:))); % solve Lyapunov equation

Cff = Tf*Caa*transpose(Tf) + Rf*Omega*transpose(Rf);
Cyy = Z*Caa*transpose(Z) + H*Omega*transpose(H);
Cyf = Z*Ta*Caa*transpose(Tf) + Z*Ra*Omega*transpose(Rf);
Cya = Z*Caa;
Cfa = Tf*Caa*transpose(Ta) + Rf*Omega*transpose(Ra);

C = [Cyy,Cyf,Cya;transpose(Cyf),Cff,Cfa;transpose(Cya),transpose(Cfa),Caa];

TT = [Z*Ta;Tf;Ta];
for i = 1 : order
  C(:,:,i+1) = TT*C(ny+nf+1:end,:,i);
end

if ~isempty(Za)
  for i = 0 : order
    C(ny+nf+1:end,:,i+1) = Za*C(ny+nf+1:end,:,i+1);
    C(:,ny+nf+1:end,i+1) = C(:,ny+nf+1:end,i+1)*transpose(Za);
  end
end
diffuse = [dy,df,db];
C(diffuse,:,:) = Inf;
C(:,diffuse,:) = Inf;

end % of primary function -----------------------------------------------------------------------------------