function Phi = srf(T,R,K,Z,H,D,Za,Omega,nper)
%
% SRF  Shock response function (or VMA representation) of general state space.
%
% Syntax:
%   Phi = srf(T,R,K,Z,H,D,Za,Omega,nper)
% Arguments:
%   Phi numeric; T,R,K,Z,H,D,Za,Omega numeric; nper numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

ny = size(Z,1);
[nx,nb] = size(T);
nf = nx - nb;
ne = size(R,2);

Phi = zeros([ny+nf+nb,ne,nper]);
Phi(:,:,1) = [H;R];
if ny > 0, Phi(1:ny,:,1) = Phi(1:ny,:,1) + Z*Phi(ny+nf+1:end,:,1); end
for i = 2 : nper
  Phi(ny+1:end,:,i) = T*Phi(ny+nf+1:end,:,i-1);
  if ny > 0, Phi(1:ny,:,i) = Z*Phi(ny+nf+1:end,:,i); end
end

if ~isempty(Za)
  for i = 1 : nper, Phi(ny+nf+1:end,:,i) = Za*Phi(ny+nf+1:end,:,i); end
end

end % of primary function -----------------------------------------------------------------------------------