function x = bpass(x,band,varargin)
%
% BPASS  Christiano-Fitzgerald band-pass filter.
%
% Syntax:
%   x = bpass(x,band,...)
% Arguments:
%   x numeric; band numeric
% Options:
%   'log' logical (false)
%   'detrend' logical (true)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'log',false,...
  'detrend',true,...
};
options = passopt(default,varargin{:});

if ~isnumeric(band) , error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

if band(2) <= band(1), error('Lower periodicity must be lower than upper one.'); end

if band(1) < 2
  warning('Lower periodicity reset to 2.')
  band(1) = 2;
end

fcn = iff(options.detrend == true,@detrend,@(x)x);
if options.log == true, x = log(x); end
sample0 = false([1,size(x,1)]);
for i = 1 : size(x,2)
  sample = getsample(transpose(x(:,i)));
  if any(sample ~= sample0), A = bpass_(band(1),band(2),sum(sample)); end
  x(sample,i) = A*fcn(x(sample,i));
  sample0 = sample;
end
if options.log == true, x = exp(x); end

end % of primary function -----------------------------------------------------------------------------------
  
  function AA = bpass_(pl,pu,T) % subfunction ---------------------------------------------------------------

  j = 1 : T;
  
  b = 2*pi/pl;
  a = 2*pi/pu;
  bnot = (b-a)/pi;
  bhat = bnot/2;
  
  B = ctranspose([(sin(j*b)-sin(j*a))./(j*pi)]);
  B(2:T,1) = B(1:T-1,1);
  B(1,1) = bnot;
  
  AA = zeros(2*T,2*T);
  
  for i = 1 : T
    AA(i,i:i+T-1) = ctranspose(B);
    AA(i:i+T-1,i) = B;   
  end
  AA = AA(1:T,1:T);
  
  AA(1,1) = bhat;
  AA(T,T) = bhat;
  
  for i = 1 : T-1
    AA(i+1,1) = AA(i,1)-B(i,1);
  	AA(T-i,T) = AA(i,1)-B(i,1);
  end
    
  end % of subfunction --------------------------------------------------------------------------------------