function [x,F] = hwbpf(x,band,varargin)
%
% HWBPF  Iacobucci-Noullez H-windowed band-pass filter.
%
% Syntax:
%   [x,F] = hwbpf(x,band,...)
% Arguments:
%   x numeric; F numeric; band numeric
% Options:
%   'log' logical (false)
%   'window' 'hamming'|'hanning' ('hamming')
%
% The IRIS Toolbox 2007/02/22. Copyright 2007 Jaromir Benes.

default = {...
  'log',false,...
  'window','hamming',...
};
options = passopt(default,varargin{:});

if ~isnumeric(band) , error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

if strcmp(options.window,'hanning'), a = 0.5;
  else, a = 0.53; end

if options.log == true, x = log(x); end
xtrend = trend(x);
x = x - xtrend;

band(band < 2) = 2;
lo1 = 2*pi/max(band);
hi1 = 2*pi/min(band);
lo2 = 2*pi - hi1;
hi2 = 2*pi - lo1;

nper_ = 0;
F = nan(size(x));
for i = 1 : size(x,2)
  sample = getsample(transpose(x(:,i)));
  nper = sum(sample);
  if nper == 0, continue, end
  if nper ~= nper_
    freq = transpose(2*pi*(0:nper-1)/nper);
    H = (freq > lo1 & freq < hi1) | (freq > lo2 & freq < hi2);
    W = toeplitz([a,(1-a)/2,zeros([1,nper-2])]);
    W(1,end) = (1-a)/2;
    W(end,1) = (1-a)/2;
    nper_ = nper;
  end
  F(sample,i) = W*H;
  x(sample,i) = ifft(F(sample,i).*fft(x(sample,i)));
end

% include regression line if zero frequency included in band
if max(band) == inf, x = x + xtrend; end
if options.log == true, x = exp(x); end

end % of primary function -----------------------------------------------------------------------------------