function [xsa,xsfactor,xtrend,ss] = seasf(x,varargin)
%
% SEASF  Quarterly seasonal filter.
%
% [xsa,xsfactor,xtrend] = seasf(x,...)
% Arguments:
%   xsa tseries, xsfactor tseries, xtrend tseries, x tseries
% Options:
%   'log' logical (false)
%   'stdseason' numeric (1)
%   'stdnoise' numeric (1)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'log',false,...
  'stdseason',1,...
  'stdnoise',1,...
};
options = passopt(default,varargin{:});

% -----function SEASF body----- %

if options.log == true, x = log(x); end

[nx,nper] = size(x);

Omega = diag([options.stdseason.^2,options.stdnoise.^2,1]);

T = zeros(6);
T(1,1) = -1;
T(4,4) = 1;
T(4,5) = -1;
T(5,5) = 1;
T(1:3,2:3) = [
   1.951342197643074e-001   -7.928362396760397e-001
   3.089016136371307e-001    1.642463930225863e+000
  -6.669371465326406e-001   -3.089016136371308e-001
];

R = zeros([6,3]);
R(6,2) = 1;
R(4,3) = -1;
R(1:3,1) = [
  -5.773502691896262e-001
   5.654093113015662e-001
  -5.890492147182234e-001
];

K = zeros([6,1]);

Z = zeros([1,6]);
Z(1,4) = -1;
Z(1,6) = 1;
Z(1,1:3) = [
  -5.773502691896254e-001    5.654093113015658e-001   -5.890492147182240e-001
];

D = 0;

H = zeros([1,3]);

Zp = zeros(6);
Zp(2,6) =  1;
Zp(3,4) = -1;
Zp(4,5) =  1;

Zp(1,1:3) = [
  -5.773502691896254e-001    5.654093113015658e-001   -5.890492147182240e-001
];
Zp(5:6,1:3) = [
   5.773502691896254e-001   -2.274269283744730e-001   -7.841834344825319e-001
  -5.773502691896260e-001   -7.928362396760393e-001   -1.951342197643082e-001
];

ss = sspace(T,R,K,Z,H,D,Zp,Omega);
dpack{2} = nan([6,nper]);
dpack{3} = nan([3,nper]);
dpack{4} = 1 : nper;

xsfactor = nan(size(x));
xtrend = nan(size(x));
for i = 1 : nx
  dpack{1} = x(i,:);
  [mlogl,se2,pred,smooth] = kalmanf(ss,dpack);
  xsfactor(i,:) = smooth.mean{2}(1,:);
  xtrend(i,:) = smooth.mean{2}(3,:);
end
xsa = x - xsfactor;

if options.log == true
  xsa = exp(xsa);
  xsfactor = exp(xsfactor);
  xtrend = exp(xtrend);
end

end