function [X,index,selection,notfound] = select(X,descriptor,selection)
%
% SELECT  Retrieve submatrix (covariance, correlation, power spectrum or spectral density) corresponding to a selection of variables.
%
% Syntax
%   [X,selection,index,notfound] = select(X,descriptor,selection)
% Arguments:
%   X numeric|cell; index numeric; selection cellstr|cell; notfound cellstr; descriptor cellstr; selection cellstr|char
%
% See also MODEL/ACF, MODEL/XSF, RVAR/ACF, RVAR/XSF.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if ischar(selection), selection = {selection}; end

if iscellstr(descriptor)
  descriptor = regexprep(descriptor,'log\((.*?)\)','$1');
else
  descriptor{1} = regexprep(descriptor{1},'log\((.*?)\)','$1');
  descriptor{2} = regexprep(descriptor{2},'log\((.*?)\)','$1');
end
selection = regexprep(selection,'log\((.*?)\)','$1');

if iscellstr(descriptor) && size(X,1) == size(X,2)
  if size(X,1) == length(descriptor)
    [index,notfound] = findnames(descriptor,selection);
    index(isnan(index)) = [];
    X = X(index,index,:,:);
    selection = descriptor(index);
  else
    error('Size of matrix and length of descriptor must match.');
  end
elseif iscell(descriptor) && iscellstr(descriptor{1}) && iscellstr(descriptor{2})
  if size(X,1) == length(descriptor{1}) && size(X,2) == length(descriptor{2})
    [index{1},notfound] = findnames(descriptor{1},selection);
    [index{2},notfound] = findnames(descriptor{2},notfound);
    index{1}(isnan(index{1})) = [];
    index{2}(isnan(index{2})) = [];
    X = X(index{1},index{2},:,:);
    selection = {descriptor{1}(index{1}),descriptor{2}(index{2})};
  else
    error('Size of matrix and length of descriptors must match.');
  end    
else
  error('Incorrect type of input argument(s).')
end

end % of primary function -----------------------------------------------------------------------------------