function mat = irisconfig(varargin)
%
% IRISCONFIG  Iris configuration file.
%
% Syntax:
%   c = irisconfig(...)
% Arguments:
%   c struct
% Options:
%   'freqletters' char ('YZQM')
%   'dateformat' char ('YFP')
%   'plotdateformat char ('Y:P')
%   'figureposition' numeric ([0,0,500*[1.7,1]])
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

mat = struct();
default_();
paths_();
if nargin > 0, mat = passopt(mat,varargin{:}); end
save(fullfile(mat.irisfolder,'irisconfig.mat'),'-struct','mat');

% end of function body --------------------------------------------------------------------------------------

  function default_() % nested function ---------------------------------------------------------------------

  mat.freqletters = 'YZQM';
  mat.dateformat = 'YFP';
  mat.plotdateformat = 'Y:P';
  mat.figureposition = [0,0,500*[1.7,1]];

  end % of nested function ----------------------------------------------------------------------------------

  function paths_() % nested function -----------------------------------------------------------------------

  try, run('irisxtools');
    catch, end

  if exist('x12Folder') == 1, x12Folder(find(x12Folder == '"')) = '';
    else, x12Folder = ''; end

  if exist('texbinFolder') == 1, texbinFolder(find(texbinFolder == '"')) = '';
    else, texbinFolder = ''; end

  mat.irisfolder = lower(fileparts(which('irisstartup.m')));
  mat.x12exepath = '';
  mat.x12mdlpath = '';
  mat.latexpath = '';
  mat.dvipspath = '';
  mat.dvipdfmpath  = '';

  if ~isempty(x12Folder)
    warning('off','MATLAB:rmpath:DirNotFound');
    warning('off','MATLAB:dispatcher:pathWarning');
    try
      addpath(x12Folder);
      if exist('x12a') == 2, mat.x12exepath = fullfile(x12Folder,'x12a');
        elseif exist('x12a.exe') == 2, mat.x12exepath = fullfile(x12Folder,'x12a');
        elseif exist('x12axp.exe') == 2, mat.x12exepath = fullfile(x12Folder,'x12axp'); end
      if any(exist('x12a.mdl') == [2,4]), mat.x12mdlpath = fullfile(x12Folder,'x12a.mdl'); end
      rmpath(x12Folder);
    catch, end
    warning('on','MATLAB:dispatcher:pathWarning');
    warning('on','MATLAB:rmpath:DirNotFound');
  end

  if ~isempty(texbinFolder)
    warning('off','MATLAB:dispatcher:pathWarning');
    warning('off','MATLAB:rmpath:DirNotFound');
    try
      addpath(texbinFolder);
      if exist('latex') == 2, mat.latexpath = fullfile(texbinFolder,'latex');
        elseif exist('latex.exe') == 2, mat.latexpath = fullfile(texbinFolder,'latex'); end
      if exist('dvips') == 2, mat.dvipspath = fullfile(texbinFolder,'dvips');
        elseif exist('dvips.exe') == 2, mat.dvipspath = fullfile(texbinFolder,'dvips'); end
      if exist('dvipdfm') == 2, mat.dvipdfmpath = fullfile(texbinFolder,'dvipdfm');
        elseif exist('dvipdfm.exe') == 2, mat.dvipdfmpath = fullfile(texbinFolder,'dvipdfm'); end
      if exist('ps2pdf') == 2, mat.ps2pdfpath = fullfile(texbinFolder,'ps2pdf');
        elseif exist('ps2pdf.exe') == 2, mat.ps2pdfpath = fullfile(texbinFolder,'ps2pdf'); end
      rmpath(texbinFolder);
    catch, end
    warning('on','MATLAB:dispatcher:pathWarning');
    warning('on','MATLAB:rmpath:DirNotFound');
  end

  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------
