function irisstartup(varargin)
%
% IRISSTARTUP  Add IRIS folders to the Matlab search path and link IRIS with external tools.
%
% Syntax:
%   irisstartup
%   irisstartup -noconflicts
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.


if any(strcmpi(varargin,'-nodisplay')), display = false;
  else display = true; end

if any(strcmpi(varargin,'-noconflicts')) || any(strcmpi(varargin,'-noconflict')), conflicts = false;
  else conflicts = true; end

if exist('display') ~= 1, display = true; end

% function body ---------------------------------------------------------------------------------------------

config = irisconfig();

% remove all existing IRIS folders from the Matlat search path
run(fullfile(config.irisfolder,'irisremove'));

% get the whole iris folder structure
addpathlist = genpath(config.irisfolder);
% remove the folder "conflicts" if required
if conflicts == false, addpathlist = regexprep(addpathlist,'(?<=^|;)([^;]*iris[^;]*conflicts[^;]*);','','ignorecase'); end
% add iris folders to the Matlab search path
addpath(addpathlist);

% check for presence of x12 and latex utils
if display == true
  disp(sprintf('  %s ready. Copyright %s Jaromir Benes.',irisping,datestr(now,'YYYY')));
  % x12
  if isempty(config.x12exepath)
    disp('  IRIS Warning: Census X12-ARIMA not linked. Unable to use TSERIES/X12 function.');
  elseif isempty(config.x12mdlpath)
    disp('  IRIS Warning: Automodel for Census X12-ARIMA not linked. Unable to use TSERIES/X12 function with ''arima'' option.');
  end  
  % latex 
  if isempty(config.latexpath)
    disp('  IRIS Warning: LaTeX not linked. Unable to use REPORT/COMPILE function.');
  else
    if isempty(config.dvipspath)
      disp('  IRIS Warning: DVIPS not found. Unable to use REPORT/COMPILE function witd PS extensions.');
    end
    if isempty(config.dvipdfmpath)
      disp('  IRIS Warning: DVIPDFM not found. Unable to use REPORT/COMPILE function with PDF extensions.');
    end
  end
  disp(' ');
end

% compile seasf model
seasffolder = fullfile(config.irisfolder,'utils/plain/private');
seasfname = fullfile(seasffolder,'seasf.model');
if exist(seasfname) == 2
  m = model(seasfname,'linear',true);
  m = solve(m);
  savestruct(fullfile(seasffolder,'seasf.mat'),m);
elseif display == true
  disp('  IRIS Warning: SEASF seasonal adjustment model not found. Unable to use TSERIES/SEASF function.');
end

end % of primary function -----------------------------------------------------------------------------------